﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;
using System.Windows.Markup;

namespace ContentMenuDemo
{
    public class ListBoxContextMenu : ContextMenu
    {
        private ListBox listbox;
        private MouseEventArgs mouseEvent;
        public ListBoxContextMenu(ListBox listbox, MouseEventArgs mouseEvent)
        {
            this.listbox = listbox;
            this.mouseEvent = mouseEvent;
        }

        public void Show()
        {
            this.Show(this.mouseEvent.GetPosition(null));
        }

        protected override FrameworkElement GetContent()
        {
            //將內容用DataTemplate方式寫在Resources中。
            //因為DataTemplate沒有辦法加入事件，所以讀出來後在添加。
            FrameworkElement template = (Application.Current.Resources["ListBoxContextMenu"] as DataTemplate).LoadContent() as FrameworkElement;
            (template.FindName("InsertButton") as Button).Click += new RoutedEventHandler(OnInsertButtonClick);
            (template.FindName("EditButton") as Button).Click += new RoutedEventHandler(OnEditButtonClick);
            (template.FindName("DeleteButton") as Button).Click += new RoutedEventHandler(OnDeleteButtonClick);

            return template;
        }

        private void OnInsertButtonClick(object sender, RoutedEventArgs e)
        {
            this.Close();

            RenameWindow window = new RenameWindow();
            window.Closed += (x, y) =>
            {
                if (window.DialogResult.Value)
                {
                    int index = GetIndex(GetChild<VirtualizingStackPanel>(listbox).Children, mouseEvent, true, false);
                    if (index == int.MaxValue)
                    {
                        this.listbox.Items.Add(window.Value);
                    }
                    else
                    {
                        this.listbox.Items.Insert(index, window.Value);
                    }
                }

            };
            window.Show();

        }

        private void OnEditButtonClick(object sender, RoutedEventArgs e)
        {
            this.Close();

            int index = GetIndex(GetChild<VirtualizingStackPanel>(listbox).Children, mouseEvent, false, false);

            if (index != int.MaxValue && index != int.MinValue)
            {
                RenameWindow window = new RenameWindow();
                window.Closed += (x, y) =>
                {
                    if (window.DialogResult.Value)
                    {
                        this.listbox.Items[index] = window.Value;
                    }

                };
                window.Value = this.listbox.Items[index] as string;
                window.Show();
            }
        }

        private void OnDeleteButtonClick(object sender, RoutedEventArgs e)
        {
            this.Close();

            int index = GetIndex(GetChild<VirtualizingStackPanel>(listbox).Children, mouseEvent, false, false);

            if (index != int.MaxValue && index != int.MinValue)
            {
                this.listbox.Items.RemoveAt(index);
            }
        }

        public static T GetChild<T>(DependencyObject element) where T : UIElement
        {
            if (element is T)
            {
                return element as T;
            }
            else
            {
                int index = VisualTreeHelper.GetChildrenCount(element);
                DependencyObject temp;
                for (int i = 0; i < index; i++)
                {
                    temp = VisualTreeHelper.GetChild(element, i);
                    if ((temp = GetChild<T>(temp)) is T)
                    {
                        return temp as T;
                    }
                }
            }

            return null;
        }


        public static int GetIndex(UIElementCollection collection, MouseEventArgs e, bool overTopAsFirst = true, bool overBottomAsLast = true)
        {
            // 超出第一筆是否為第一筆
            int index = overTopAsFirst ? 0 : int.MinValue;
            int count = collection.Count;

            if (count > 0)
            {
                ///找出目前在那一個項目上
                FrameworkElement item;
                Point p;

                p = e.GetPosition(collection[0]);

                if (p.Y > 0)
                {
                    item = collection[count - 1] as FrameworkElement;
                    p = e.GetPosition(item);

                    if (p.Y > item.ActualHeight)
                    {
                        // 超出最後一筆是否為最後一筆
                        index = overBottomAsLast ? count - 1 : int.MaxValue;
                    }
                    else
                    {
                        index = 0;
                        for (int i = 1; i < count; i++)
                        {
                            item = collection[i] as FrameworkElement;
                            p = e.GetPosition(item);
                            if (p.Y >= 0 && p.Y <= item.ActualHeight)
                            {
                                index = i;
                                break;
                            }
                        }
                    }
                }
            }

            return index;
        }
    }
}